@@ -1,13 +1,14 @@
 /*
 	libmpg123: MPEG Audio Decoder library
 
-	copyright 1995-2007 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1995-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 
 */
 
 #include "mpg123lib_intern.h"
 #include "getbits.h"
+#include "debug.h"
 
 #ifdef GAPLESS
 #define SAMPLE_ADJUST(x)   ((x) - ((mh->p.flags & MPG123_GAPLESS) ? mh->begin_os : 0))
@@ -786,7 +787,7 @@ off_t mpg123_feedseek(mpg123_handle *mh, off_t sampleoff, int whence, off_t *inp
 	mh->buffer.fill = 0;
 
 	/* Shortcuts without modifying input stream. */
-	*input_offset = mh->rdat.firstpos + mh->rdat.filelen;
+	*input_offset = mh->rdat.buffer.fileoff + mh->rdat.buffer.size;
 	if(mh->num < mh->firstframe) mh->to_decode = FALSE;
 	if(mh->num == pos && mh->to_decode) goto feedseekend;
 	if(mh->num == pos-1) goto feedseekend;
