@@ -1,7 +1,7 @@
 /*
 	parse: spawned from common; clustering around stream/frame parsing
 
-	copyright ?-2007 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright ?-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp & Thomas Orgis
 */
@@ -9,7 +9,6 @@
 #include "mpg123lib_intern.h"
 
 #include <sys/stat.h>
-#include <sys/time.h>
 #include <fcntl.h>
 
 #include "getbits.h"
@@ -18,6 +17,18 @@
 #include <winsock.h>
 #endif
 
+/* a limit for number of frames in a track; beyond that unsigned long may not be enough to hold byte addresses */
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
+#ifndef ULONG_MAX
+/* hm, is this portable across preprocessors? */
+#define ULONG_MAX ((unsigned long)-1)
+#endif
+#define TRACK_MAX_FRAMES ULONG_MAX/4/1152
+
+#include "debug.h"
+
 #define bsbufid(fr) (fr)->bsbuf==(fr)->bsspace[0] ? 0 : ((fr)->bsbuf==fr->bsspace[1] ? 1 : ( (fr)->bsbuf==(fr)->bsspace[0]+512 ? 2 : ((fr)->bsbuf==fr->bsspace[1]+512 ? 3 : -1) ) )
 
 /*
@@ -68,15 +79,6 @@ static const int tabsel_123[2][3][16] = {
 
 const long freqs[9] = { 44100, 48000, 32000, 22050, 24000, 16000 , 11025 , 12000 , 8000 };
 
-/* a limit for number of frames in a track; beyond that unsigned long may not be enough to hold byte addresses */
-#ifdef HAVE_LIMITS_H
-#include <limits.h>
-#endif
-#ifndef ULONG_MAX
-/* hm, is this portable across preprocessors? */
-#define ULONG_MAX ((unsigned long)-1)
-#endif
-#define TRACK_MAX_FRAMES ULONG_MAX/4/1152
 
 #ifdef VARMODESUPPORT
 	/*
@@ -134,9 +136,9 @@ int head_check(unsigned long head)
 		||
 		/* 1111 means bad bitrate */
 		(((head>>12)&0xf) == 0xf)
-/*		|| */
-		/* 0000 means free format... */
-/*		(((head>>12)&0xf) == 0x0) */
+		||
+		/* 0000 means free format... which should be supported in future. */
+		(((head>>12)&0xf) == 0x0)
 		||
 		/* sampling freq: 11 is reserved */
 		(((head>>10)&0x3) == 0x3 )
@@ -597,7 +599,11 @@ init_resync:
       }
 
       if(give_note && NOQUIET && (newhead & 0xffffff00) == ('b'<<24)+('m'<<16)+('p'<<8)) fprintf(stderr,"Note: Could be a BMP album art.\n");
-      if (!(fr->p.flags & MPG123_NO_RESYNC) || fr->do_recover) {
+      /* Do resync if forced from outside or not forbidden by flag.
+          Also, don't try to resync on ICY streams - that won't work! */
+      if( (!(fr->p.flags & MPG123_NO_RESYNC)|| fr->do_recover)
+           && fr->p.icy_interval == 0 )
+      {
         long try = 0;
         long limit = fr->p.resync_limit;
         /* TODO: make this more robust, I'd like to cat two mp3 fragments together (in a dirty way) and still have mpg123 beign able to decode all it somehow. */
@@ -616,7 +622,7 @@ init_resync:
             if(give_note && NOQUIET) fprintf (stderr, "Note: Hit end of (available) data during resync.\n");
             goto read_frame_bad;
           }
-          debug3("resync try %i at 0x%lx, got newhead 0x%08lx", try, (unsigned long)fr->rd->tell(fr),  newhead);
+          debug3("resync try %li at 0x%lx, got newhead 0x%08lx", try, (unsigned long)fr->rd->tell(fr),  newhead);
           if (!fr->oldhead)
           {
             debug("going to init_resync...");
@@ -626,19 +632,25 @@ init_resync:
          /* Michael's new resync routine seems to work better with the one frame readahead (and some input buffering?) */
          } while
          (
-              (newhead & HDRCMPMASK) != (fr->oldhead & HDRCMPMASK)
-           && (newhead & HDRCMPMASK) != (fr->firsthead & HDRCMPMASK)
+           !head_check(newhead) /* Simply check for any valid header... we have the readahead to get it straight now(?) */
+           /*   (newhead & HDRCMPMASK) != (fr->oldhead & HDRCMPMASK)
+           && (newhead & HDRCMPMASK) != (fr->firsthead & HDRCMPMASK)*/
          );
          /* too many false positives 
          }while (!(head_check(newhead) && decode_header(fr, newhead))); */
+        if(give_note && NOQUIET) fprintf (stderr, "Note: Skipped %li bytes in input.\n", try);
          if(limit >= 0 && try >= limit)
          {
            if(NOQUIET) error1("Giving up resync after %li bytes - your stream is not nice... (maybe increasing resync limit could help).", try);
            fr->err = MPG123_RESYNC_FAIL;
            return READER_ERROR;
          }
-
-        if(give_note && NOQUIET) fprintf (stderr, "Note: Skipped %li bytes in input.\n", try);
+         else
+         {
+           debug1("Found valid header 0x%lx... unsetting firsthead to reinit stream.", newhead);
+           fr->firsthead = 0;
+           goto init_resync;
+         }
       }
       else
       {
@@ -667,7 +679,7 @@ init_resync:
     fr->header_change = 0;
 
 	/* if filepos is invalid, so is framepos */
-	framepos = fr->rdat.filepos - 4;
+	framepos = fr->rd->tell(fr) - 4;
 	/* flip/init buffer for Layer 3 */
 	{
 		unsigned char *newbuf = fr->bsspace[fr->bsnum]+512;
@@ -740,6 +752,7 @@ init_resync:
 	if(fr->error_protection) fr->crc = getbits(fr, 16); /* skip crc */
 	return 1;
 read_frame_bad:
+	if(fr->err == MPG123_OK) fr->err = MPG123_ERR_READER;
 	fr->framesize = oldsize;
 	fr->halfphase = oldphase;
 	return ret;
@@ -953,9 +966,9 @@ int mpg123_position(mpg123_handle *fr, off_t no, off_t buffsize,
 #if 0
 	curs = curs < 0 ? 0.0 : curs;
 #endif
-	if(lefts < 0)
-	{
-		if(NOQUIET) warning("seconds_left < 0!");
+	if(left < 0 || lefts < 0)
+	{ /* That is the case for non-seekable streams. */
+		left  = 0;
 		lefts = 0.0;
 	}
 	if(current_frame != NULL) *current_frame = cur;
